# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ClientFacingWorkoutDurationSampleIntensity(str, enum.Enum):
    LOW = "low"
    MEDIUM = "medium"
    HIGH = "high"

    def visit(
        self,
        low: typing.Callable[[], T_Result],
        medium: typing.Callable[[], T_Result],
        high: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ClientFacingWorkoutDurationSampleIntensity.LOW:
            return low()
        if self is ClientFacingWorkoutDurationSampleIntensity.MEDIUM:
            return medium()
        if self is ClientFacingWorkoutDurationSampleIntensity.HIGH:
            return high()
