# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ClientFacingForcedVitalCapacityChangedEventType(str, enum.Enum):
    DAILY_DATA_FORCED_VITAL_CAPACITY_CREATED = "daily.data.forced_vital_capacity.created"
    DAILY_DATA_FORCED_VITAL_CAPACITY_UPDATED = "daily.data.forced_vital_capacity.updated"

    def visit(
        self,
        daily_data_forced_vital_capacity_created: typing.Callable[[], T_Result],
        daily_data_forced_vital_capacity_updated: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ClientFacingForcedVitalCapacityChangedEventType.DAILY_DATA_FORCED_VITAL_CAPACITY_CREATED:
            return daily_data_forced_vital_capacity_created()
        if self is ClientFacingForcedVitalCapacityChangedEventType.DAILY_DATA_FORCED_VITAL_CAPACITY_UPDATED:
            return daily_data_forced_vital_capacity_updated()
