# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ClientFacingBodyMassIndexChangedEventType(str, enum.Enum):
    DAILY_DATA_BODY_MASS_INDEX_CREATED = "daily.data.body_mass_index.created"
    DAILY_DATA_BODY_MASS_INDEX_UPDATED = "daily.data.body_mass_index.updated"

    def visit(
        self,
        daily_data_body_mass_index_created: typing.Callable[[], T_Result],
        daily_data_body_mass_index_updated: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ClientFacingBodyMassIndexChangedEventType.DAILY_DATA_BODY_MASS_INDEX_CREATED:
            return daily_data_body_mass_index_created()
        if self is ClientFacingBodyMassIndexChangedEventType.DAILY_DATA_BODY_MASS_INDEX_UPDATED:
            return daily_data_body_mass_index_updated()
