# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ClientFacingWorkoutStreamChangedEventType(str, enum.Enum):
    DAILY_DATA_WORKOUT_STREAM_CREATED = "daily.data.workout_stream.created"
    DAILY_DATA_WORKOUT_STREAM_UPDATED = "daily.data.workout_stream.updated"

    def visit(
        self,
        daily_data_workout_stream_created: typing.Callable[[], T_Result],
        daily_data_workout_stream_updated: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ClientFacingWorkoutStreamChangedEventType.DAILY_DATA_WORKOUT_STREAM_CREATED:
            return daily_data_workout_stream_created()
        if self is ClientFacingWorkoutStreamChangedEventType.DAILY_DATA_WORKOUT_STREAM_UPDATED:
            return daily_data_workout_stream_updated()
