# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ClientFacingPeakExpiratoryFlowRateChangedEventType(str, enum.Enum):
    DAILY_DATA_PEAK_EXPIRATORY_FLOW_RATE_CREATED = "daily.data.peak_expiratory_flow_rate.created"
    DAILY_DATA_PEAK_EXPIRATORY_FLOW_RATE_UPDATED = "daily.data.peak_expiratory_flow_rate.updated"

    def visit(
        self,
        daily_data_peak_expiratory_flow_rate_created: typing.Callable[[], T_Result],
        daily_data_peak_expiratory_flow_rate_updated: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ClientFacingPeakExpiratoryFlowRateChangedEventType.DAILY_DATA_PEAK_EXPIRATORY_FLOW_RATE_CREATED:
            return daily_data_peak_expiratory_flow_rate_created()
        if self is ClientFacingPeakExpiratoryFlowRateChangedEventType.DAILY_DATA_PEAK_EXPIRATORY_FLOW_RATE_UPDATED:
            return daily_data_peak_expiratory_flow_rate_updated()
