# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ClientFacingStandHourSampleType(str, enum.Enum):
    STOOD = "stood"
    IDLE = "idle"

    def visit(self, stood: typing.Callable[[], T_Result], idle: typing.Callable[[], T_Result]) -> T_Result:
        if self is ClientFacingStandHourSampleType.STOOD:
            return stood()
        if self is ClientFacingStandHourSampleType.IDLE:
            return idle()
